@@ -48,7 +48,7 @@
 #include "polChemDefEntity.hpp"
 #include "sequence.hpp"
 #include "mzCalculationDlg.hpp"
-#include "isotopicPatternCalculationDlg.hpp"
+#include "spectrumCalculationDlg.hpp"
 
 
 namespace massXpert
@@ -1164,9 +1164,16 @@ namespace massXpert
   void 
   CalculatorWnd::isotopicPatternCalculation()
   {
-    IsotopicPatternCalculationDlg *dlg = 
-      new IsotopicPatternCalculationDlg(this, m_polChemDef.atomList());
+    SpectrumCalculationDlg *dlg = 
+      new SpectrumCalculationDlg(this, 
+                                 m_polChemDef.atomList(),
+                                 MXP_SPECTRUM_CALCULATION_MODE_CLUSTER);
   
+    // We really need that the dialog be set modal because we want to
+    // be able to access all the atom and oligomer data freely and
+    // durably.
+    dlg->setModal(true);
+    
     dlg->show();
   }
 
