@@ -45,6 +45,13 @@
 #include "cleavageDlg.hpp"
 #include "application.hpp"
 
+#include "peakShape.hpp"
+#include "peakCentroid.hpp"
+#include "peakShapeConfig.hpp"
+#include "isotopicPatternCalculator.hpp"
+#include "spectrumCalculationDlg.hpp"
+
+
 namespace massXpert
 {
 
@@ -113,7 +120,8 @@ namespace massXpert
     comboBoxItemList 
       << tr("To Clipboard") 
       << tr("To File")
-      << tr("Select File");
+      << tr("Select File")
+      << tr("Calculate spectrum");
   
     m_ui.exportResultsComboBox->addItems(comboBoxItemList);
   
@@ -386,7 +394,7 @@ bool
     cleaveOptions.setIonizeLevels(valueStart, valueEnd);
     
     // The list in which we'll store all the allocated oligomers.
-    OligomerList oligomerList;
+    OligomerList tempOligomerList;
     
     delete mpa_cleaver;
     mpa_cleaver = 0;
@@ -402,7 +410,7 @@ bool
     mpa_cleaver= new Cleaver(mp_polymer, mp_polChemDef, cleaveOptions,
 			     m_calcOptions, *mp_ionizeRule);
 
-    mpa_cleaver->setOligomerList(&oligomerList);
+    mpa_cleaver->setOligomerList(&tempOligomerList);
     
     if (!mpa_cleaver->cleave())
       {
@@ -416,7 +424,7 @@ bool
       }
 
     // At this point we have a brand new set of oligomers in the local
-    // list of oligomers (oligomerList). We either stack these on top
+    // list of oligomers (tempOligomerList). We either stack these on top
     // of the previous ones, or we replace the previous ones with the
     // new ones.  Are we stacking new oligomers on top of the old
     // ones?
@@ -429,17 +437,21 @@ bool
         
     // At this point we can set up the data to the treeview model.
 
-    int initialOligomers = oligomerList.size();
+    int initialOligomers = tempOligomerList.size();
 
     // qDebug() << __FILE__ << __LINE__
     //          << "initialOligomers:" << initialOligomers;
  
-    // We are *transferring* the oligomers from oligomerList to the
-    // list of oligomers that is in the model : we are not duplicating
-    // the oligomers. When the transfer has been done, oligomerList
-    // will be empty.
+    // We are *transferring* the oligomers from tempOligomerList to
+    // the list of m_oligomerList list oligomers of which there is a
+    // pointer in the model : we are not duplicating the
+    // oligomers. When the transfer from tempOligomerList to
+    // m_oligomerList _via_ the model will have been done,
+    // tempOligomerList will be empty and m_oligomerList will hold
+    // them all, with the model having been made aware of that with
+    // the beginInsertRows/endInsertRows statement pair.
     int addedOligomers = 
-      mpa_oligomerTableViewModel->addOligomers(oligomerList);
+      mpa_oligomerTableViewModel->addOligomers(tempOligomerList);
 
     // qDebug() << __FILE__ << __LINE__
     //          << "addedOligomers:" << addedOligomers;
@@ -448,9 +460,9 @@ bool
       qFatal("Fatal error at %s@%d. Aborting.", __FILE__, __LINE__);
     
     // Now that we have *transferred* (not copied) all the oligomers
-    // in the model, we can clear the oligomerList without freeing the
+    // in the model, we can clear the tempOligomerList without freeing the
     // instances it currently contain...
-    oligomerList.clear();
+    tempOligomerList.clear();
     
     // Set focus to the treeView.
     m_ui.oligomerTableView->setFocus();
@@ -475,8 +487,8 @@ bool
     // Update the cleavage details so that we know how the oligos were
     // computed.
     updateCleavageDetails(m_calcOptions);
-}
-
+  }
+  
 
   void
   CleavageDlg::updateCleavageDetails(const CalcOptions &calcOptions)
@@ -787,6 +799,10 @@ bool
       {
 	selectResultsFile();
       }
+    else if (index == 3)
+      {
+	calculateSpectrum();
+      }
     else 
       Q_ASSERT(0);
   
@@ -872,6 +888,30 @@ bool
 
     return true;
   }
+
+
+  bool
+  CleavageDlg::calculateSpectrum()
+  {
+    SpectrumCalculationDlg *dlg = 
+      new SpectrumCalculationDlg(this, 
+                                 mp_polChemDef->atomList(),
+                                 MXP_SPECTRUM_CALCULATION_MODE_SPECTRUM);
+
+    // We now have to prepare the land for the calculations.
+
+    dlg->setOligomerList(&m_oligomerList);
+
+    // We really need that the dialog be set modal because we want to
+    // be able to access all the atom and oligomer data freely and
+    // durably.
+    dlg->setModal(true);
+    
+    dlg->show();
+
+    return true;
+  }
+  
   //////////////////////////////////// The results-exporting functions.
   //////////////////////////////////// The results-exporting functions.
   //////////////////////////////////// The results-exporting functions.
