@@ -534,6 +534,12 @@ namespace massXpert
       m_ui.rightModifCheckBox->setCheckState(Qt::Checked);
     else
       m_ui.rightModifCheckBox->setCheckState(Qt::Unchecked);
+
+    if (calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
+      m_ui.crossLinksCheckBox->setCheckState(Qt::Checked);
+    else
+      m_ui.crossLinksCheckBox->setCheckState(Qt::Unchecked);
+
   }
 
 
