@@ -82,7 +82,7 @@ namespace massXpert
 
     populateToleranceTypeComboBoxes();
   
-    setupTreeViews();
+    setupTableViews();
   
     m_ui.ionizationStartSpinBox->setRange(-1000000000, 1000000000);
     m_ui.ionizationStartSpinBox->setValue(1);
@@ -191,9 +191,9 @@ namespace massXpert
     m_ui.oligoDetailsSplitter->
       restoreState(settings.value("oligoDetailsSplitter").toByteArray());
   
-    m_ui.oligoDetailsSplitter->
-      restoreState(settings.value("treeviewsSplitter").toByteArray());
-  
+    m_ui.tableViewsSplitter->
+      restoreState(settings.value("tableViewsSplitter").toByteArray());
+    
     settings.endGroup();
 
 
@@ -271,11 +271,11 @@ namespace massXpert
   {
     delete mpa_resultsString;
   
-    delete mpa_monoOligomerTreeViewModel;
+    delete mpa_monoOligomerTableViewModel;
   
     delete mpa_monoProxyModel;
 
-    delete mpa_avgOligomerTreeViewModel;
+    delete mpa_avgOligomerTableViewModel;
   
     delete mpa_avgProxyModel;
 
@@ -303,8 +303,8 @@ namespace massXpert
     settings.setValue("oligoDetailsSplitter", 
 		       m_ui.oligoDetailsSplitter->saveState());
 
-    settings.setValue("treeviewsSplitter", 
-		       m_ui.oligoDetailsSplitter->saveState());
+    settings.setValue("tableViewsSplitter", 
+		       m_ui.tableViewsSplitter->saveState());
 
     settings.endGroup();
   }
@@ -453,7 +453,6 @@ namespace massXpert
   bool
   MassSearchDlg::checkTolerance(int type)
   {
-    int index = 0;
     double nominal = 0;
     bool ok = false;
 
@@ -461,12 +460,10 @@ namespace massXpert
 
     if (type & MXT_MASS_MONO)
       {
-	index = m_ui.monoToleranceComboBox->currentIndex();
 	text = m_ui.monoMassLineEdit->text();
       }
     else if (type & MXT_MASS_AVG)
       {
-	index = m_ui.avgToleranceComboBox->currentIndex();
 	text = m_ui.avgMassLineEdit->text();
       }
     else
@@ -648,7 +645,7 @@ namespace massXpert
 		localIonizeRule.setLevel(kter);
 	      
 		Oligomer oligomerTest(mp_polymer, "NOT_SET", Ponderable(),
-				       localIonizeRule, false,
+                                      localIonizeRule, CalcOptions(),false,
 				       iter, iter + jter);
 	    
 		oligomerTest.calculateMasses(&m_calcOptions);
@@ -702,8 +699,8 @@ namespace massXpert
 		oligomerNew->appendProp(prop);
 	      
 		type & MXT_MASS_MONO ? 
-		  mpa_monoOligomerTreeViewModel->addOligomer(oligomerNew) :
-		  mpa_avgOligomerTreeViewModel->addOligomer(oligomerNew);
+		  mpa_monoOligomerTableViewModel->addOligomer(oligomerNew) :
+		  mpa_avgOligomerTableViewModel->addOligomer(oligomerNew);
 	      
 		++m_foundOligosCount;
 	      
@@ -795,8 +792,8 @@ namespace massXpert
     updateIonizationData();
   
     // Empty the treeviews and the lists of oligomers.
-    mpa_monoOligomerTreeViewModel->removeAll();
-    mpa_avgOligomerTreeViewModel->removeAll();
+    mpa_monoOligomerTableViewModel->removeOligomers(0, -1);
+    mpa_avgOligomerTableViewModel->removeOligomers(0, -1);
   
     emptyAllMassLists();
 
@@ -878,7 +875,7 @@ namespace massXpert
 	   QMessageBox::Ok);
 
 	setCursor(Qt::ArrowCursor);
-	updateMassSearchDetails();
+	updateMassSearchDetails(m_calcOptions);
 	application->beep();
 
 	return;
@@ -896,7 +893,7 @@ namespace massXpert
 	   QMessageBox::Ok);
 
 	setCursor(Qt::ArrowCursor);
-	updateMassSearchDetails();
+	updateMassSearchDetails(m_calcOptions);
 	application->beep();
 
 	return;
@@ -904,37 +901,39 @@ namespace massXpert
   
     setCursor(Qt::ArrowCursor);
 
-    updateMassSearchDetails();
+    updateMassSearchDetails(m_calcOptions);
 
     application->beep();
   }
 
 
   void
-  MassSearchDlg::setupTreeViews()
+  MassSearchDlg::setupTableViews()
   {
     // Model stuff all thought for sorting.
-    mpa_monoOligomerTreeViewModel = 
-      new MassSearchOligomerTreeViewModel(&m_monoOligomerList, this);
+    mpa_monoOligomerTableViewModel = 
+      new MassSearchOligomerTableViewModel(&m_monoOligomerList, this);
 
-    mpa_avgOligomerTreeViewModel = 
-      new MassSearchOligomerTreeViewModel(&m_avgOligomerList, this);
+    mpa_avgOligomerTableViewModel = 
+      new MassSearchOligomerTableViewModel(&m_avgOligomerList, this);
   
-    mpa_monoProxyModel = new MassSearchOligomerTreeViewSortProxyModel(this);
-    mpa_monoProxyModel->setSourceModel(mpa_monoOligomerTreeViewModel);
+    mpa_monoProxyModel = new MassSearchOligomerTableViewSortProxyModel(this);
+    mpa_monoProxyModel->setSourceModel(mpa_monoOligomerTableViewModel);
     mpa_monoProxyModel->setFilterKeyColumn(-1);
 
-    mpa_avgProxyModel = new MassSearchOligomerTreeViewSortProxyModel(this);
-    mpa_avgProxyModel->setSourceModel(mpa_avgOligomerTreeViewModel);
+    mpa_avgProxyModel = new MassSearchOligomerTableViewSortProxyModel(this);
+    mpa_avgProxyModel->setSourceModel(mpa_avgOligomerTableViewModel);
     mpa_avgProxyModel->setFilterKeyColumn(-1);
 
-    m_ui.monoOligomerTreeView->setModel(mpa_monoProxyModel);
-    m_ui.monoOligomerTreeView->setParentDlg(this);
-    mpa_monoOligomerTreeViewModel->setTreeView(m_ui.monoOligomerTreeView);
+    m_ui.monoOligomerTableView->setModel(mpa_monoProxyModel);
+    m_ui.monoOligomerTableView->setParentDlg(this);
+    m_ui.monoOligomerTableView->setOligomerList(&m_monoOligomerList);
+    mpa_monoOligomerTableViewModel->setTableView(m_ui.monoOligomerTableView);
 
-    m_ui.avgOligomerTreeView->setModel(mpa_avgProxyModel);
-    m_ui.avgOligomerTreeView->setParentDlg(this);
-    mpa_avgOligomerTreeViewModel->setTreeView(m_ui.avgOligomerTreeView);
+    m_ui.avgOligomerTableView->setModel(mpa_avgProxyModel);
+    m_ui.avgOligomerTableView->setParentDlg(this);
+    m_ui.avgOligomerTableView->setOligomerList(&m_avgOligomerList);
+    mpa_avgOligomerTableViewModel->setTableView(m_ui.avgOligomerTableView);
   }
 
 
@@ -1014,22 +1013,22 @@ namespace massXpert
 
   
   void
-  MassSearchDlg::updateMassSearchDetails()
+  MassSearchDlg::updateMassSearchDetails(const CalcOptions &calcOptions)
   {
-    if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
-      m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_MODIF)
+      m_ui.modificationsCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.modifsPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.modificationsCheckBox->setCheckState(Qt::Unchecked);
 
-    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
-      m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_LEFT_END_MODIF)
+      m_ui.leftModifCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.leftModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.leftModifCheckBox->setCheckState(Qt::Unchecked);
 
-    if (m_calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
-      m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/greenled.png"));
+    if (calcOptions.polymerEntities() & MXT_POLYMER_CHEMENT_RIGHT_END_MODIF)
+      m_ui.rightModifCheckBox->setCheckState(Qt::Checked);
     else
-      m_ui.rightModifPixmapLabel->setPixmap(QPixmap(":/images/redled.png"));
+      m_ui.rightModifCheckBox->setCheckState(Qt::Unchecked);
   }
 
 
@@ -1516,9 +1515,6 @@ namespace massXpert
     *mpa_resultsString += QObject::tr("\n---------------------------\n"
 				       "Mass Search: \n"
 				       "---------------------------\n");
-  
-    bool monoSearched = true;
-    bool avgSearched = true;
 
     // Should the oligomers have their sequence displayed?
     bool withSequence = m_ui.withSequenceCheckBox->isChecked();
@@ -1530,13 +1526,12 @@ namespace massXpert
 	*mpa_resultsString += QObject::tr("\nSearched mono masses:\n"
 					   "%1\n")
 	  .arg(masses);
-	monoSearched = false;
       }
   
     // The mono treeview:
 
     QString *text = 
-      m_ui.monoOligomerTreeView->selectedOligomersAsPlainText(withSequence);
+      m_ui.monoOligomerTableView->selectedOligomersAsPlainText(withSequence);
 
     *mpa_resultsString += *text;
     
@@ -1549,12 +1544,11 @@ namespace massXpert
 	*mpa_resultsString += QObject::tr("\nSearched average masses:\n"
 					   "%1\n")
 	  .arg(masses);
-	avgSearched = false;
       }
   
     // The avg treeview:
     
-    text = m_ui.avgOligomerTreeView->selectedOligomersAsPlainText(withSequence);
+    text = m_ui.avgOligomerTableView->selectedOligomersAsPlainText(withSequence);
 
     *mpa_resultsString += *text;
     
