@@ -655,7 +655,6 @@ namespace massXpert
     // in the polymer chemistry definition, or perform a quick and dirty
     // modification definition in the dialog.
 
-    bool fromPolChemDef = false;
     QString text;
 
     Modif *modif =  new Modif(polymer->polChemDef(), 
@@ -706,7 +705,6 @@ namespace massXpert
 
 	// At this point the modification is correct. Set a bool to true
 	// to know later that the modif was from the polChemDef.
-	fromPolChemDef = true;
       }
   
     // Is the modification work with allowed overriding of the target
