@@ -43,6 +43,7 @@
 #include "mzCalculationDlg.hpp"
 #include "application.hpp"
 #include "polChemDef.hpp"
+#include "spectrumCalculationDlg.hpp"
 
 
 namespace massXpert
@@ -52,7 +53,8 @@ namespace massXpert
 				     const PolChemDef *polChemDef,
 				     const IonizeRule *ionizeRule,
 				     double mono, double avg)
-    : QDialog(parent), mp_polChemDef(polChemDef)
+    : QDialog(parent), mp_polChemDef(polChemDef),
+      m_atomList(mp_polChemDef->atomList())
   {
     Q_ASSERT(parent);
     Q_ASSERT(polChemDef);
@@ -74,6 +76,21 @@ namespace massXpert
     if (avg != 0)
       m_ui.avgLineEdit->setText(locale.toString(avg, 'f', 
 						MXP_OLIGOMER_DEC_PLACES));
+
+    if(mono || avg)
+      {
+        // If we are setting masses directly from here, then that means
+        // that the formula line edit must be disabled.
+        m_ui.formulaCheckBox->setCheckState(Qt::Unchecked);
+        m_ui.startFormulaLineEdit->setDisabled(true);
+      }
+    else
+      {
+        // No masses are being preseeded. So let the user the full
+        // opportunity ot enter a formula.
+        m_ui.formulaCheckBox->setCheckState(Qt::Checked);
+        m_ui.startFormulaLineEdit->setDisabled(false);
+      }
     
     m_ui.srcIonizeRuleChargeSpinBox->setRange(1, 1000000000);
     m_ui.srcIonizeRuleLevelSpinBox->setRange(0, 1000000000);
@@ -100,7 +117,8 @@ namespace massXpert
     comboBoxItemList 
       << tr("To Clipboard") 
       << tr("To File")
-      << tr("Select File");
+      << tr("Select File")
+      << tr("Calculate spectrum");
   
     m_ui.exportResultsComboBox->addItems(comboBoxItemList);
   
@@ -131,6 +149,11 @@ namespace massXpert
 	     SIGNAL(clicked()),
 	     this,
 	     SLOT(calculate()));
+
+    connect(m_ui.formulaCheckBox,
+            SIGNAL(toggled(bool)),
+            this,
+            SLOT(formulaCheckBoxToggled(bool)));
   }
 
 
@@ -143,6 +166,9 @@ namespace massXpert
     delete mpa_mzProxyModel;
 
     freeIonizableList();
+
+    while(!m_oligomerList.isEmpty())
+      delete m_oligomerList.takeFirst();
   }
 
 
@@ -183,16 +209,152 @@ namespace massXpert
   }
 
 
+  void
+  MzCalculationDlg::formulaCheckBoxToggled(bool checked)
+  {
+    // If checked, then the user is responsible for entering a valid
+    // formula.
+
+    if(checked)
+      {
+        m_ui.startFormulaLineEdit->setDisabled(true);
+      }
+    else
+      {
+        m_ui.startFormulaLineEdit->setDisabled(false);
+      }
+  }
+  
+ 
   void
   MzCalculationDlg::calculate()
   {
+    // The calculation can be accomplished in two different manners:
+    //
+    // 1. Starting from a formula, in which case it will be necessary
+    // to know what is the ionization status of the analyte for which
+    // the formula is entered. This status is found in the source
+    // ionizeRule. Once this is done, the masses can be computed for
+    // the formula and are programmatically set to the mono and avg
+    // m/z line edit widgets.
+    // 
+    // 2. Starting from mono and avg m/z values immediately. Uncheck
+    // the Formula check box in this case.
+    
     // First get all the data for the source ionizable.
 
-    // The source ionizeRule.
+    // Because this function can be called multiple times from the
+    // same dialog window, make sure that the formula is cleared.
+    m_formula.clear();
+    
+    // The source ionizeRule. Start with this because we'll need right
+    // away.
     IonizeRule srcIonizeRule;
-    if (!getSrcIonizeRuleData(&srcIonizeRule))
+    if(!getSrcIonizeRuleData(&srcIonizeRule))
       return;
     
+    // Are we being given a formula to start with?
+    
+    bool fromFormula = false;
+    
+    if(m_ui.formulaCheckBox->checkState() == Qt::Checked)
+      {
+        QString text = m_ui.startFormulaLineEdit->text();
+        
+        if(text.isEmpty())
+          {
+            // Then the masses must be null.
+            m_ui.monoLineEdit->setText("0.000");
+            m_ui.avgLineEdit->setText("0.000");
+            
+            return;
+          }
+        
+        Formula formula(text);
+
+        // Do not bother storing all the atoms in the formula, this is
+        // only a check (false param below).  m_atomList is a reference to
+        // the atom list of the polymer chemistry definition currently
+        // used in the caller calculator window.
+
+        if (!formula.validate(m_atomList, false, true))
+          {
+            m_ui.monoLineEdit->setText("0.000");
+            m_ui.avgLineEdit->setText("0.000");
+
+            QMessageBox::warning 
+              (this, 
+               tr("massXpert - m/z Ratio Calculator"),
+               tr("%1@%2\n"
+                  "Failed to validate the formula.")
+               .arg(__FILE__)
+               .arg(__LINE__),
+               QMessageBox::Ok);
+            
+            return;
+          }
+    
+        // qDebug() << __FILE__ << __LINE__
+        //          << "Formula:" << formula.formula() << "validated." ;
+
+        // At this point we know we could actually validate the formula,
+        // so do that work with the member formula:
+
+        // We want to validate the formula and in the mean time construct
+        // the list of all the AtomCount objects(first true), and since
+        // the formula is reused we also ensure that that list is reset
+        // (second true). m_atomList is a reference to the atom list of
+        // the polymer chemistry definition currently used in the caller
+        // calculator window.
+
+        m_formula.setFormula(text);
+        m_formula.validate(m_atomList, true, true);
+    
+        // Calculation of the m/z corresponding to the formula requires
+        // dividing the mass obtained for the formula by the product
+        // (level * unitary) of the ionizerule.
+
+        int charge = srcIonizeRule.charge() * srcIonizeRule.level();
+    
+        double monoMass = 0;
+        double avgMass = 0;
+    
+        // It is impossible that we have an error here, since the formula
+        // was previously validated.
+        if (!m_formula.accountMasses(m_atomList, &monoMass, &avgMass, 1))
+          qFatal("Fatal error at %s@%d.Aborting.", __FILE__, __LINE__);
+
+        if(charge)
+          {
+            monoMass /= charge;
+            avgMass /= charge;
+          }
+    
+        // Great, now show the data.
+
+        Application *application = static_cast<Application *>(qApp);
+    
+        QString mz;
+    
+        mz = application->locale().
+          toString(monoMass, 'f', MXP_OLIGOMER_DEC_PLACES);
+        // qDebug() << __FILE__ << __LINE__
+        //          << "monoMass:" << monoMass;
+        
+        m_ui.monoLineEdit->setText(mz);
+
+        mz = application->locale().
+          toString(avgMass, 'f', MXP_OLIGOMER_DEC_PLACES);
+        // qDebug() << __FILE__ << __LINE__
+        //          << "avgMass:" << avgMass;
+
+        m_ui.avgLineEdit->setText(mz);
+
+        fromFormula = true;
+      }
+
+    // Whatever the way we get the mon/avg mz values, make a
+    // ponderable with them.
     Ponderable *ponderable = getSourcePonderable();
     if (!ponderable)
       return;
@@ -202,14 +364,39 @@ namespace massXpert
     // level, in which case we tell so to the caller with a true
     // boolean.
     Ionizable ionizable(mp_polChemDef, "NOT_SET", *ponderable,
-			 srcIonizeRule,
+                        srcIonizeRule,
 			(srcIonizeRule.level() ? true : false));
     
     // At this point the ionizable is fully qualified.
 
     // No longer in use.
     delete ponderable;
-    
+
+    // Now, because we want to be able to compute elemental
+    // compositions for each destination ionization level, we have to
+    // kraft an elemental representation of what the molecule
+    // is. This, only if the mz values above were calculated with a
+    // provided formula.
+    if(fromFormula)
+      {
+        // The text string in the startFormulaLineEdit widget
+        // corresponds to the molecular entity already ionized with
+        // srcIonizeRule (if level is non-zero). So, to go back to
+        // non-ionized elemental composition, we have to remove from
+        // the formula the equivalent atoms of the source ionizerule
+        // multiplied by ionization level.
+
+        // qDebug() << __FILE__ << __LINE__
+        //          << "Formula before deionization:" << m_formula.text();
+        
+        m_formula.accountInList(srcIonizeRule.formula(),
+                                m_atomList, 
+                                -1 * srcIonizeRule.level());
+        
+        // qDebug() << __FILE__ << __LINE__
+        //          << "Formula after deionization:" << m_formula.text();
+      }
+        
     // At this point, the source ionizable is filled with data. And we
     // know the data are correct because everything was validated.
 
@@ -232,13 +419,18 @@ namespace massXpert
 	startLevel = temp;
       }
 
-    // Empty the treeView model, so that we can put fresh data in it.
+    // Empty the treeView model, so that we can put fresh data in
+    // it. Same for the oligomer list!
+
     mpa_mzTreeViewModel->removeAll();
-  
+
+    while(!m_oligomerList.isEmpty())
+      delete m_oligomerList.takeFirst();
+        
     for (int level = startLevel ; level < endLevel + 1; ++level)
       {
 	destIonizeRule.setLevel(level);
-	
+
 	// Reionize the ionizable.
 	if(ionizable.ionize(destIonizeRule) == -1)
 	  {
@@ -252,17 +444,51 @@ namespace massXpert
 	       QMessageBox::Ok);
 	    
 	    return;
-	  }
-	else
-	  {
-	    Ionizable *newIonizable = 
-	      new Ionizable(ionizable);
-	    
-	    mpa_mzTreeViewModel->addIonizable(newIonizable);
-	  }
+          }
+        
+        Ionizable *newIonizable = new Ionizable(ionizable);
+        
+        mpa_mzTreeViewModel->addIonizable(newIonizable);
+        
+	// Kraft an oligomer, that we'll set apart in case the user
+	// wants to compute a spectrum to simulate the charge envelope
+	// peaks.
+
+        Oligomer *oligomer = new Oligomer(ionizable,
+                                          CalcOptions(),
+                                          -1, -1);
+
+        // qDebug() << __FILE__ << __LINE__
+        //          << "oligomer->charge:" << oligomer->charge();
+
+        // If we are working from a formula, then compute the new
+        // formula for the present ionization status and create a
+        // property with that new formula to append to the list of
+        // properties in the oligomer. 
+
+        if(fromFormula)
+          {
+            Formula newFormula(m_formula);
+            
+            newFormula.accountInList(destIonizeRule.formula(),
+                                     m_atomList, destIonizeRule.level());
+            
+            // qDebug() << __FILE__ << __LINE__
+            //          << "Formula after reionization (level)" 
+            //          << level
+            //          << newFormula.text();
+            
+            
+            StringProp *prop = new StringProp("ELEMENTAL_COMPOSITION",
+                                              newFormula.text());
+        
+            oligomer->appendProp(static_cast<Prop *>(prop));
+          }
+                
+	m_oligomerList.append(oligomer);
       }
   }
-
+  
 
   bool
   MzCalculationDlg::getSrcIonizeRuleData(IonizeRule *ionizeRule)
@@ -417,6 +643,10 @@ namespace massXpert
       {
 	selectResultsFile();
       }
+    else if (index == 3)
+      {
+	calculateSpectrum();
+      }
     else 
       Q_ASSERT(0);
   
@@ -610,6 +840,30 @@ namespace massXpert
 
     return true;
   }
+
+
+  bool
+  MzCalculationDlg::calculateSpectrum()
+  {
+    SpectrumCalculationDlg *dlg = 
+      new SpectrumCalculationDlg(this, 
+                                 mp_polChemDef->atomList(),
+                                 MXP_SPECTRUM_CALCULATION_MODE_SPECTRUM);
+    
+    // We now have to prepare the land for the calculations.
+
+    dlg->setOligomerList(&m_oligomerList);
+        
+    // We really need that the dialog be set modal because we want to
+    // be able to access all the atom and oligomer data freely and
+    // durably.
+    dlg->setModal(true);
+    
+    dlg->show();
+
+    return true;
+  }
+ 
   //////////////////////////////////// The results-exporting functions.
   //////////////////////////////////// The results-exporting functions.
   //////////////////////////////////// The results-exporting functions.
