@@ -173,7 +173,7 @@ namespace massXpert
   
     while (m_noCleaveIndexList.size())
       m_noCleaveIndexList.removeFirst();
-  
+    
     return true;
   }
 
@@ -183,7 +183,6 @@ namespace massXpert
   {
     QList<CleaveMotif *> *motifList = m_cleaveOptions.motifList();
   
-
     while (m_cleaveIndexList.size())
       m_cleaveIndexList.removeFirst();
   
@@ -237,23 +236,23 @@ namespace massXpert
 	    if (cleavageIndex >= endIndex)
 	      break;
 	  
-// 	    qDebug() << __FILE__ << __LINE__ 
-// 		      << "Found new cleavage index:"
-// 		      << cleavageIndex;
+            // qDebug() << __FILE__ << __LINE__ 
+            //          << "Found new cleavage index:"
+            //          << cleavageIndex;
 	    
 	    if (cleaveMotif->isForCleave())
 	      {
 		m_cleaveIndexList.append(cleavageIndex);
 		
-// 		qDebug() << __FILE__ << __LINE__ 
-// 			  << "For cleavage, index:" << cleavageIndex;
+                // qDebug() << __FILE__ << __LINE__ 
+                //          << "For cleavage, index:" << cleavageIndex;
 	      }
 	    else
 	      {
 		m_noCleaveIndexList.append(cleavageIndex);
 		
-// 		qDebug() << __FILE__ << __LINE__ 
-// 			  << "Not for cleavage";
+		// qDebug() << __FILE__ << __LINE__ 
+                //          << "Not for cleavage";
 	      }
 	    
 	  }
@@ -456,7 +455,7 @@ namespace massXpert
 	    // The formula has to be there.
 	    
 	    Formula ruleFormula = cleaveRule->leftFormula();
-	    Q_ASSERT(!ruleFormula.formula().isEmpty());
+	    Q_ASSERT(!ruleFormula.text().isEmpty());
 	    
 	    // What is the monomer at the left end of current oligomer ?
 	    const Monomer &monomer = oligomer->atLeftEnd();
@@ -488,7 +487,7 @@ namespace massXpert
 	    // The formula has to be there.
       
 	    Formula ruleFormula = cleaveRule->rightFormula();
-	    Q_ASSERT(!ruleFormula.formula().isEmpty());
+	    Q_ASSERT(!ruleFormula.text().isEmpty());
       
 	    // What is the monomer at the right end of current oligomer ?
 	    const Monomer &monomer = oligomer->atRightEnd();
@@ -690,12 +689,12 @@ namespace massXpert
 
 	    CleaveRule *cleaveRule = m_cleaveOptions.ruleList()->at(jter);
 	  
-// 	    qDebug() << __FILE__ << __LINE__
-// 		      << "Accounting for cleaverule:"
-// 		      << cleaveRule->name();
+	    // qDebug() << __FILE__ << __LINE__
+	    //           << "Accounting for cleaverule:"
+	    //           << cleaveRule->name();
 	    
-// 	    qDebug() << __FILE__ << __LINE__
-// 		      << "Oligomer mono mass before:" << iterOligomer->mono();
+	    // qDebug() << __FILE__ << __LINE__
+	    //           << "Oligomer mono mass before:" << iterOligomer->mono();
 	    
 	    if (!accountCleaveRule(cleaveRule, iterOligomer))
 	      return -1;
@@ -755,7 +754,9 @@ namespace massXpert
       }
 
     // Finally transfer all the oligomers generated for this partial
-    // cleavage to the list of ALL the oligomers.
+    // cleavage to the list of ALL the oligomers. But before making
+    // the transfert, compute the elemental composition and store it
+    // as a property object.
 
     int oligomerCount = partialOligomerList.size();
     
@@ -763,6 +764,13 @@ namespace massXpert
       {
 	Oligomer *iterOligomer = partialOligomerList.takeFirst();
       
+        QString *formula = new QString(iterOligomer->elementalComposition());
+        
+        StringProp *prop = new StringProp("ELEMENTAL_COMPOSITION",
+                                          formula);
+        
+        iterOligomer->appendProp(static_cast<Prop *>(prop));
+        
 	mp_oligomerList->append(iterOligomer);
       }
     
@@ -826,22 +834,27 @@ namespace massXpert
 	  }
 	else
 	  {
-	    // 	  qDebug() << __FILE__ << __LINE__ 
-	    // 		    << "Cross-link at index" << iter
-	    // 		    << "did not find any oligomer for its first monomer "
+	    // qDebug() << __FILE__ << __LINE__ 
+	    //    << "Cross-link at index" << iter
+	    //    << "did not find any oligomer for its first monomer "
 	    // 	    "partner";
 	  }
       }
   
-    // At this point we have terminated analyzing all the oligomers for
-    // the partial cleavage. All we have to do is move all the
+    // At this point we have terminated analyzing all the oligomers
+    // for the partial cleavage. All we have to do is move all the
     // crossLinked oligomers from the crossLinkedOligomerList to
-    // oligomerList.
+    // oligomerList. While doing so make sure that the m_calcOptions
+    // datum has correct CoordinateList data, as these data will be
+    // required later, typically to calculate the elemental formula of
+    // the oligomer.
 
     while (crossLinkedOligomerList.size())
       {
 	Oligomer *oligomer = crossLinkedOligomerList.takeAt(0);
-      
+
+        oligomer->updateCalcOptions();
+              
 	oligomerList->append(oligomer);
       }
   
@@ -915,30 +928,30 @@ namespace massXpert
 
 		if (!oligomer->addCrossLink(crossLink))
 		  {
-		    // 		  qDebug() << __FILE__ << __LINE__ 
-		    // 			    << "The cross-link:" 
-		    // 			    << crossLink->name() 
-		    // 			    << "was already in the"
-		    // 			    << oligomer 
-		    // 			    << "oligomer's list of cross-links: " 
+		    //  qDebug() << __FILE__ << __LINE__ 
+		    //	    << "The cross-link:" 
+		    //	    << crossLink->name() 
+		    //	    << "was already in the"
+		    //	    << oligomer 
+		    //	    << "oligomer's list of cross-links: " 
 		    // 		    "not duplicated.";
 		  }
 		else
 		  {
-		    // 		  qDebug() << __FILE__ << __LINE__ 
-		    // 			    << "The cross-link:" 
-		    // 			    << crossLink->name() 
-		    // 			    << "was added to the"
-		    // 			    << oligomer 
-		    // 			    << "oligomer's list of cross-links.";
+		    //  qDebug() << __FILE__ << __LINE__ 
+		    //	    << "The cross-link:" 
+		    //	    << crossLink->name() 
+		    //	    << "was added to the"
+		    //	    << oligomer 
+		    //	    << "oligomer's list of cross-links.";
 		  }
 
 		const Monomer *iterMonomer = 0;
 	      
 		foreach(iterMonomer, *(crossLink->monomerList()))
 		  {
-		    // 		  qDebug() << __FILE__ << __LINE__ 
-		    // 			    << iterMonomer->name();
+		    // 	 qDebug() << __FILE__ << __LINE__ 
+		    // 	    << iterMonomer->name();
 		  
 		    int foundIndex = 0;
 
@@ -948,8 +961,8 @@ namespace massXpert
 		  
 		    if (foundOligomer)
 		      {
-			// 		      qDebug() << __FILE__ << __LINE__ 
-			// 				<< foundOligomer->name() << foundIndex;
+			// qDebug() << __FILE__ << __LINE__ 
+			// 	<< foundOligomer->name() << foundIndex;
 
 			// One oligomer in the original oligomer list
 			// encompasses a monomer that seems to be
@@ -1036,30 +1049,30 @@ namespace massXpert
 
 		    if (!oligomer->addCrossLink(crossLink))
 		      {
-			// 		      qDebug() << __FILE__ << __LINE__ 
-			// 				<< "The cross-link:" 
-			// 				<< crossLink->name() 
-			// 				<< "was already in the"
-			// 				<< oligomer 
-			// 				<< "oligomer's list of cross-links: " 
-			// 			"not duplicated.";
+			// qDebug() << __FILE__ << __LINE__ 
+			// << "The cross-link:" 
+			// << crossLink->name() 
+			// << "was already in the"
+			// << oligomer 
+			// << "oligomer's list of cross-links: " 
+			// "not duplicated.";
 		      }
 		    else
 		      {
-			// 		      qDebug() << __FILE__ << __LINE__ 
-			// 				<< "The cross-link:" 
-			// 				<< crossLink->name() 
-			// 				<< "was added to the"
-			// 				<< oligomer 
-			// 				<< "oligomer's list of cross-links.";
+			// qDebug() << __FILE__ << __LINE__ 
+			// << "The cross-link:" 
+			// << crossLink->name() 
+			// << "was added to the"
+			// << oligomer 
+			// << "oligomer's list of cross-links.";
 		      }
 		  
 		    const Monomer *iterMonomer = 0;
 
 		    foreach(iterMonomer, *(crossLink->monomerList()))
 		      {
-			// 		      qDebug() << __FILE__ << __LINE__ 
-			// 				<< iterMonomer->name();
+			// qDebug() << __FILE__ << __LINE__ 
+			//	<< iterMonomer->name();
 		      
 			int foundIndex = 0;
 		      
@@ -1069,8 +1082,8 @@ namespace massXpert
 		  
 			if (foundOligomer)
 			  {
-			    // 			  qDebug() << __FILE__ << __LINE__ 
-			    // 				    << foundOligomer->name() << foundIndex;
+                            // qDebug() << __FILE__ << __LINE__ 
+                            //          << foundOligomer->name() << foundIndex;
 			  
 			    // One oligomer in the original oligomer list
 			    // encompasses a monomer that seems to be
