@@ -146,6 +146,187 @@ namespace massXpert
     //  	    << "number of fragmentation specifications:" 
     //  	    << m_fragOptionList.size();
   
+    // Before starting the calculation we ought to know if there are
+    // cross-links in the oligomer to be fragmented and if the user
+    // has asked that these cross-linked be taken into account during
+    // the fragmentation.
+
+    if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
+      {
+        qDebug() << __FILE__ << __LINE__
+                 << "Fragmentation calculations take "
+          "into account the cross-links";
+        
+        // Let's put one of the fragmentation options (the first) so that
+        // we can get the indices of the oligomer to fragment.
+        
+        FragOptions *fragOptions = m_fragOptionList.at(0);
+
+        // If there are cross-links, then we have to deal with
+        // them. The strategy is to first get a list of all the
+        // monomer indices for the monomers in the oligomer (being
+        // fragmented) that are involved in cross-links.
+
+        QList<int> crossLinkedMonomerIndexList;
+        int partials = 0;
+        
+        qDebug() << __FILE__ << __LINE__
+                 << "Fragmentation options:"
+                 << fragOptions->startIndex() << fragOptions->endIndex();
+        
+        mp_polymer->crossLinkedMonomerIndexList(fragOptions->startIndex(),
+                                                fragOptions->endIndex(),
+                                                &crossLinkedMonomerIndexList,
+                                                &partials);
+        
+        if(partials)
+          qDebug() << __FILE__ << __LINE__
+                   << "Fragmentation calculations do not\n"
+            "take into account partial cross-links.\n"
+            "These partial cross-links are ignored.";
+        
+        // Now that we have a list of the indices of all the monomers
+        // that are cross-linked, we can iterate in it and create a
+        // list of CrossLinkedRegion instances that will allow us to
+        // "segment" the to-fragment oligomer so as to ease the
+        // calculation of product ion masses.
+
+        // Sort the indices.
+        qSort(crossLinkedMonomerIndexList.begin(), 
+              crossLinkedMonomerIndexList.end());
+
+        // qDebug() << __FILE__ << __LINE__
+        //          << "Indices:" << crossLinkedMonomerIndexList;
+        
+        
+        // Now find continuous regions and create a new region each
+        // time we find one.
+
+        int first = 0;
+        int last = 0;
+        
+        int prev = 0;
+        int next = 0;
+        
+        for(int iter = 0, size = crossLinkedMonomerIndexList.size() - 1;
+            iter < size; ++iter)
+          {
+            // Seed the system only at the first iteration.
+            if(!iter)
+              {
+                first = crossLinkedMonomerIndexList.at(iter);
+                last = crossLinkedMonomerIndexList.at(iter + 1);
+                
+                // qDebug() << __FILE__ << __LINE__
+                //          << "Seeding with first and last:"
+                //          << first << "--" << last;
+              }
+            
+            prev = crossLinkedMonomerIndexList.at(iter);
+            next = crossLinkedMonomerIndexList.at(iter + 1);
+
+            if(next - prev == 1)
+              {
+                // We are going on with a continuum. Fine.
+                last = next;
+
+                // qDebug() << __FILE__ << __LINE__
+                //          << "Elongating continuum:"
+                //          << "[" << first << "-" << last << "]";
+
+                continue;
+              }
+            else
+              {
+                // There is a gap. Close the previous continuum and
+                // start another one.
+                
+                last = prev;
+                
+                // qDebug() << __FILE__ << __LINE__
+                //          << "Closing continuum:"
+                //          << "[" << first << "-" << last << "]";
+                
+                CrossLinkedRegion *region = new CrossLinkedRegion(first, last);
+                
+                // Get the cross-links for the region.
+                QList<CrossLink *> crossLinkList;
+
+                partials = 0;
+
+                mp_polymer->crossLinkList(first, last, 
+                                          &crossLinkList, &partials);
+                
+                if(partials)
+                  qDebug() << __FILE__ << __LINE__
+                           << "Fragmentation calculations do not\n"
+                    "take into account partial cross-links.\n"
+                    "These partial cross-links are ignored.";
+                
+                // Append the obtained cross-links to the region so
+                // that we finalize its construction. Finally append
+                // the new region to the list.
+
+                region->appendCrossLinks(crossLinkList);
+                
+                m_crossLinkedRegionList.append(region);
+                
+                // Now that we have closed a continuum, start seeding
+                // a new one.
+                first = next;
+              }
+          }
+        // End of 
+        // for(int iter = 0, size = crossLinkedMonomerIndexList.size() - 1;
+        //    iter < size; ++iter)
+
+        // We have to close the last continuum that we could not close
+        // because we ended off the for loop.
+        
+        last = next;
+        
+        // qDebug() << __FILE__ << __LINE__
+        //          << "Closing continuum:"
+        //          << "[" << first << "-" << last << "]";
+
+        CrossLinkedRegion *region = new CrossLinkedRegion(first, last);
+                
+        // Get the cross-links for the region.
+        QList<CrossLink *> crossLinkList;
+
+        partials = 0;
+
+        mp_polymer->crossLinkList(first, last, 
+                                  &crossLinkList, &partials);
+                
+        if(partials)
+          qDebug() << __FILE__ << __LINE__
+                   << "Fragmentation calculations do not\n"
+            "take into account partial cross-links.\n"
+            "These partial cross-links are ignored.";
+        
+        // Append the obtained cross-links to the region so
+        // that we finalize its construction. Finally append
+        // the new region to the list.
+
+        region->appendCrossLinks(crossLinkList);
+                
+        m_crossLinkedRegionList.append(region);
+
+        // qDebug() << __FILE__ << __LINE__
+        //          << "Having" << m_crossLinkedRegionList.size() << "regions";
+
+        // for(int iter = 0; iter < m_crossLinkedRegionList.size(); ++iter)
+        //   qDebug() << __FILE__ << __LINE__
+        //            << "Start:" << m_crossLinkedRegionList.at(iter)->startIndex()
+        //            << "End:" << m_crossLinkedRegionList.at(iter)->endIndex();
+      }
+    // End of
+    // if (m_calcOptions.monomerEntities() & MXT_MONOMER_CHEMENT_CROSS_LINK)
+    
+    // At this point we have a list of regions that we'll be able to
+    // use to compute the fragment masses.
+
     // For each fragmentation options instance in the list, perform the
     // required fragmentation.
 
@@ -534,6 +715,28 @@ namespace massXpert
 
     const QList<Atom *> &refList = mp_polChemDef->atomList();
 
+    // If the crosslinks are to be taken into account, then make a
+    // local copy of the m_crossLinkedRegionList because we are going
+    // to remove items from it during the calculation of the fragments
+    // and we do not want to modify the contents of the original list
+    // (remember that this fragmenter might be created to perform more
+    // than one single fragmentation but a set of fragmentations).
+
+    QList<CrossLinkedRegion *> crossLinkedRegionList;
+    
+    for(int iter = 0; iter < m_crossLinkedRegionList.size(); ++iter)
+      {
+        CrossLinkedRegion *region = 
+          new CrossLinkedRegion(*m_crossLinkedRegionList.at(iter));
+        
+        crossLinkedRegionList.append(region);
+      }
+
+    // At this point we can start making the calculations of the
+    // fragments. Because we are generating fragments that contain the
+    // left part of the oligomer, we iterate in the
+    // fragOptions.startIndex() --> fragOptions.endIndex() direction.
+
     for (int iter = fragOptions.startIndex(); 
 	 iter < fragOptions.endIndex(); ++iter, ++number)
       {
@@ -542,6 +745,54 @@ namespace massXpert
 	const Monomer *monomer = mp_polymer->at(iter);
       
 	monomer->accountMasses(&ponderable, 1);
+
+
+        // If we are to take into account the cross-links, we ought to
+        // take them into account here *once* and then remove them
+        // from the crossLinkedRegionList so that we do not take them
+        // into account more than once.
+
+        // Iterate in the crossLinkedRegionList (do that in reverse
+        // order because we'll have at some point to have to remove
+        // items) and...
+
+        int jter = crossLinkedRegionList.size() - 1;
+
+        while(jter >= 0)
+          {
+            // ... for each item in it ask if the region encompasses
+            // the current monomer index (value of iter)....
+            
+            CrossLinkedRegion *region = crossLinkedRegionList.at(jter);
+            
+            if(region->endIndex() == iter)
+              {
+                // ... if so, iterate in the list of cross-links that
+                // is stored in the CrossLinkedRegion...
+                
+                const QList<CrossLink *> & crossLinkList =
+                  region->crossLinkList();
+                
+                for(int kter = 0; kter < crossLinkList.size(); ++kter)
+                  {
+                    // ... and for each cross-link, account its mass
+                    // in the fragment (that is, ponderable)...
+
+                    CrossLink *crossLink = crossLinkList.at(kter);
+                    
+                    crossLink->accountMasses(&ponderable, 1);
+                  }
+
+                // ... and remove+delete the CrossLinkedRegion from
+                // the list so that we are sure we do not take that
+                // cross-link into account more than once.
+
+                delete crossLinkedRegionList.takeAt(jter);
+              }
+            
+            --jter;
+          }
+        
       
 	Ponderable ponderableTemp(ponderable);
       
@@ -843,7 +1094,8 @@ namespace massXpert
               while(!ionizeLevelOligomerList->isEmpty())
                 {
                   FragmentOligomer *iterOligomer = 
-                    static_cast<FragmentOligomer *>(ionizeLevelOligomerList->takeFirst());
+                    static_cast<FragmentOligomer *>
+                    (ionizeLevelOligomerList->takeFirst());
                   
                   mp_oligomerList->append(iterOligomer);
                   ++count;
@@ -882,6 +1134,28 @@ namespace massXpert
   
     const QList<Atom *> &refList = mp_polChemDef->atomList();
 
+    // If the crosslinks are to be taken into account, then make a
+    // local copy of the m_crossLinkedRegionList because we are going
+    // to remove items from it during the calculation of the fragments
+    // and we do not want to modify the contents of the original list
+    // (remember that this fragmenter might be created to perform more
+    // than one single fragmentation but a set of fragmentations).
+
+    QList<CrossLinkedRegion *> crossLinkedRegionList;
+    
+    for(int iter = 0; iter < m_crossLinkedRegionList.size(); ++iter)
+      {
+        CrossLinkedRegion *region = 
+          new CrossLinkedRegion(*m_crossLinkedRegionList.at(iter));
+        
+        crossLinkedRegionList.append(region);
+      }
+
+    // At this point we can start making the calculations of the
+    // fragments. Because we are generating fragments that contain the
+    // right part of the oligomer, we iterate in the
+    // fragOptions.endIndex() --> fragOptions.startIndex() direction.
+
     for (int iter = fragOptions.endIndex(); 
 	 iter > fragOptions.startIndex(); --iter, ++number)
       {
@@ -891,6 +1165,53 @@ namespace massXpert
       
 	monomer->accountMasses(&ponderable, 1);
       
+        // If we are to take into account the cross-links, we ought to
+        // take them into account here *once* and then remove them
+        // from the crossLinkedRegionList so that we do not take them
+        // into account more than once.
+
+        // Iterate in the crossLinkedRegionList (do that in reverse
+        // order because we'll have at some point to have to remove
+        // items) and...
+
+        int jter = crossLinkedRegionList.size() - 1;
+
+        while(jter >= 0)
+          {
+            // ... for each item in it ask if the region encompasses
+            // the current monomer index (value of iter)....
+            
+            CrossLinkedRegion *region = crossLinkedRegionList.at(jter);
+            
+            if(region->startIndex() == iter)
+              {
+                // ... if so, iterate in the list of cross-links that
+                // is stored in the CrossLinkedRegion...
+                
+                const QList<CrossLink *> & crossLinkList =
+                  region->crossLinkList();
+                
+                for(int kter = 0; kter < crossLinkList.size(); ++kter)
+                  {
+                    // ... and for each cross-link, account its mass
+                    // in the fragment (that is, ponderable)...
+
+                    CrossLink *crossLink = crossLinkList.at(kter);
+                    
+                    crossLink->accountMasses(&ponderable, 1);
+                  }
+
+                // ... and remove+delete the CrossLinkedRegion from
+                // the list so that we are sure we do not take that
+                // cross-link into account more than once.
+
+                delete crossLinkedRegionList.takeAt(jter);
+              }
+            
+            --jter;
+          }
+        
+
 	Ponderable ponderableTemp(ponderable);
       
 	if(!fragOptions.formula().isEmpty())    
@@ -1502,7 +1823,7 @@ namespace massXpert
         QString newName = name;
 
         newName.append(QString("#%1#z=%2")
-                       .arg(formula->formula())
+                       .arg(formula->text())
                        .arg(charge));
                         
         newOligomer->setName(newName);
