@@ -56,7 +56,7 @@ namespace massXpert
     \param formula Formula.
   */
   Monomer::Monomer(const PolChemDef *polChemDef, QString name, 
-		    QString code, QString formula)
+                   QString code, QString formula)
     : PolChemDefEntity(polChemDef, name), 
       Formula(formula), Ponderable(0, 0),
       m_code(code)
@@ -83,9 +83,9 @@ namespace massXpert
   Monomer::clone() const
   {
     Monomer *other = new Monomer(mp_polChemDef,
-				  m_name,
-				  m_code,
-				  m_formula);
+                                 m_name,
+                                 m_code,
+                                 m_formula);
     Ponderable::clone(other);
 
     if (mpa_modifList)
@@ -372,8 +372,8 @@ namespace massXpert
   */
   int 
   Monomer::isCodeInList(const QString &str, 
-			 const QList<Monomer*> &refList,
-			 Monomer *other)
+                        const QList<Monomer*> &refList,
+                        Monomer *other)
   {
     Monomer *monomer = 0;
   
@@ -440,8 +440,8 @@ namespace massXpert
   */
   int 
   Monomer::isNameInList(const QString &str, 
-			 const QList<Monomer*> &refList,
-			 Monomer *other)
+                        const QList<Monomer*> &refList,
+                        Monomer *other)
   {
     Monomer *monomer = 0;
   
@@ -473,6 +473,13 @@ namespace massXpert
   }
 
 
+  QString
+  Monomer::formula() const
+  {
+    return Formula::text();
+  }
+  
+
   bool 
   Monomer::isModifTarget(const Modif &modif) const
   {
